
#include "sssd.h"
#include "internals.h"


/* Internal generic load function - note 'mode' contains flags */

int sssd_copySampleStereo( struct sssdaudio* ga, void* data, int channels, int freq, int bits, int samples, int mode )
{
    struct ga_sample* sample;
    int id;
    int i, sample1,counter;
    unsigned char *cursample;
    unsigned char *inc, *outc;
    short *ins, *outs;
    void *dataL;

#ifndef __NOPRINTING__
        printf( "sssd_copySample...(): enter\n" );
#endif

    ga->lastERR=0;

    if( ! audioValid( ga ) ) {
        ga->lastERR=ERR_NOT_INITIALIZED;
        return( ERR_NOT_INITIALIZED );
    }
    /* Check entry count */
    if( ga->num_samples >= ga->max_samples )
    {
#ifndef __NOPRINTING__
        printf( "sssd: sample list is full\n" );
#endif
 	ga->lastERR=ERR_SAMPLES_FULL;
        return( ERR_SAMPLES_FULL );
    }

    id     = ga->num_samples;
    sample = &ga->samples[ id ];

    dataL=data;    

#if 0 /* turn on resampling */
    /* Resample if necessary and requested to do so */
#ifndef __NOPRINTING__
printf("%x\n freq=%d  ga->freq=%d\n",mode,freq,ga->freq);
#endif
    if(!(mode&COPY_DONT_RESAMPLE)) {
      if(ga->freq!=freq) {
#ifndef __NOPRINTING__
printf("Resampling\n");
#endif
         dataL=malloc((int)((float)ga->freq/freq)*samples);
#ifndef __NOPRINTING__
puts("1");
#endif
         samples=sssd_resampleRaw(ga,data, freq,samples,channels,bits,dataL,ga->freq, mode&RESAMPLE_MASK);
#ifndef __NOPRINTING__
puts("2");
#endif
	 freq=ga->freq;
      }
    }
#endif

    /* Lock buffers while loading sample */
    LOCK_BUFFERS(ga);

    /* Setup sample length */
    sample->numsamples = samples;

#ifndef __NOPRINTING__
printf("sample bits:%d    subsystem bits:%d\n",bits,ga->bits);
printf("sample chan:%d    subsystem chan:%d\n",channels,ga->channels);
#endif

    /* Create buffer */
    sample->data = malloc( ga->byps * samples );
    if( sample->data==NULL ) {
#ifndef __NOPRINTING__
        puts( "sssd_copySample: error allocating sample memory\n" );
#endif
        UNLOCK_BUFFERS(ga);
	ga->lastERR=ERR_SAMPLES_ALLOC;
	return ERR_SAMPLES_ALLOC;
    }
    sample->flags = 0;

    if(ga->bits==bits && ga->channels==channels) {
       /* Just copy the data, no need to convert */
#ifndef __NOPRINTING__
        puts( "sssd_copySample: copy with no conversion" );
#endif
       memcpy(sample->data,data,samples*ga->byps);
    } else {
#ifndef __NOPRINTING__
        puts( "sssd_copySample: copy conversion" );
#endif
    
    
        /* Copy data with conversion */
	ins=(short *)dataL;
	inc=(unsigned char *)dataL;
        outs=(short *)sample->data;
        outc=(unsigned char *)sample->data;
	
        for(counter=0;counter<samples;counter++) {
	   
	   if(bits==16) sample1=*ins;
	   else sample1=*inc;
	   
	   if(channels==2 && ga->channels==1) {
	      ins++;inc++;
	      if(bits==16) sample1+=*ins;
	      else sample1+=*inc;
	      sample1>>=1;
	   }
	   
	   if(ga->bits==16 && bits==8) sample1=sample8to16(sample1);
	   else
	   if(ga->bits==8 && bits==16) sample1=sample16to8(sample1);
	   
	   if(ga->bits==16) *outs=sample1;
	   else *outc=sample1;
	   
	   if(channels==1 && ga->channels==2) {
	      outs++;outc++;
	      if(ga->bits==16) *outs=sample1;
	      else *outc=sample1;
	   }
	   ins++;inc++;
	   outs++;outc++;
	}

    } /* copy with/without conversion */

        /* Blank one or the other channel depending on mode */
       if(ga->channels!=1) {
        if (mode == COPY_LEFT) {
#ifndef __NOPRINTING__
        puts( "sssd_copySample: COPY_LEFT" );
#endif
    	 cursample=(unsigned char *)sample->data;
    	 if (ga->byps==2) {
     	  for (i=0; i<sample->numsamples; i++) {
    	   *cursample=0;
    	   cursample += 2;
    	  }
    	 } else if (ga->byps==4) {
     	  for (i=0; i<sample->numsamples; i++) {
    	   *cursample=0;
    	   cursample += 1;
    	   *cursample=0;
    	   cursample += 3;
    	  }
    	 }
        } else if (mode == COPY_RIGHT) {
#ifndef __NOPRINTING__
        puts( "sssd_copySample: COPY_RIGHT" );
#endif
    	 cursample=(unsigned char *)sample->data;
    	 if (ga->byps==2) {
    	 cursample++;
     	  for (i=0; i<sample->numsamples; i++) {
    	   *cursample=0;
    	   cursample += 2;
    	  }
    	 } else if (ga->byps==4) {
     	  cursample += 2;
     	  for (i=0; i<sample->numsamples; i++) {
    	   *cursample=0;
    	   cursample += 1;
    	   *cursample=0;
    	   cursample += 3;
    	  }
    	 }
        
	} /* COPY_LEFT or COPY_RIGHT */
       } /* Channel blanking */

    
    
    /* Calculate end-of-sample position pointer */ 
    sample->dataendmarker = (char *)sample->data + (ga->byps * samples);

    ga->num_samples++;
    

    UNLOCK_BUFFERS(ga);
    if(dataL!=data) free(dataL);

    return( id );
}
