#include "sssd.h"
#include "internals.h"
#include "resample/stdefs.h"
#include "resample/resample.h"


int SrcLinear(HWORD X[], HWORD Y[], double factor, UWORD *Time, UHWORD Nx);
int SrcUp(HWORD X[], HWORD Y[], double factor, UWORD *Time,
                 UHWORD Nx, UHWORD Nwing, UHWORD LpScl,
                 HWORD Imp[], HWORD ImpD[], BOOL Interp);
int SrcUD(HWORD X[], HWORD Y[], double factor, UWORD *Time,
                 UHWORD Nx, UHWORD Nwing, UHWORD LpScl,
                 HWORD Imp[], HWORD ImpD[], BOOL Interp);


int sssd_resampleRaw(struct sssdaudio *ga, void *insamp, int infreq, int insize, int inchannels, int inbits, void *outdata, int outfreq, int flags) {
  
  unsigned int outCount,inBytes, outBytes;
  double factor;
  unsigned char *inC, *outC;
  short *inS, *outS, *ItmpS, *OtmpS, *Ltmp, *Rtmp;
  int sample, i;
  BOOL interpFilt;
  
  UHWORD LpScl;               /* Unity-gain scale factor */
  UHWORD Nwing;               /* Filter table size */
  UHWORD Nmult;               /* Filter length for up-conversions */
  HWORD *Imp=0;               /* Filter coefficients */
  HWORD *ImpD=0;              /* ImpD[n] = Imp[n+1]-Imp[n] */
  UWORD Time, Time2;          /* Current time/pos in input sample */

  #include "resample/smallfilter.h"

  Nmult = SMALL_FILTER_NMULT;
  Imp = (short *)SMALL_FILTER_IMP;         /* Impulse response */
  ImpD = (short *)SMALL_FILTER_IMPD;       /* Impulse response deltas */
  LpScl = SMALL_FILTER_SCALE;     /* Unity-gain scale factor */
  Nwing = SMALL_FILTER_NWING;     /* Filter table length */

    
  ga->lastERR=0;

  inBytes=insize*inchannels;
  if(inbits==16) inBytes<<=1;
     
  factor=(float)outfreq/(float)infreq;

#ifndef __NOPRINTING__
  printf("resample in: %d   out:%d   factor: %f\n",infreq, outfreq, factor);
#endif
  
  /* Don't need to resample if infreq==outfreq, just copy */
  if(factor==1) {
     memcpy(outdata,insamp,inBytes);
     return insize;  
  }
  
  /* setup a temp buffer */
     Ltmp=ItmpS=(short *)malloc(((insize*inchannels)<<1)+20);
     inS=(short *)insamp;
     inC=(unsigned char *)inS;

     for(i=0;i<9;i++) *Ltmp++=0x00;
     Rtmp=Ltmp+(int)((float)insize*factor)+11;
     outCount=insize;
     while(outCount>0) {
        
	if(inbits==8) sample=sample8to16(*inC);
	else sample=*inS;
	*Ltmp=sample;
	inC++;inS++;

	if(inchannels==2) {
	  if(inbits==8) sample=sample8to16(*inC);
	  else sample=*inS;
	  *Rtmp=sample;
	  inC++;inS++;
        }
	Ltmp++;Rtmp++;
     
     outCount--;
     }
  
  
  OtmpS=(short *)malloc(((float)i*factor)+2);
  Ltmp=ItmpS+10;
  Rtmp=Ltmp+(int)((float)insize*factor)+11;
  Time2 = Time = 10 << Np;
  
  if(flags&RESAMPLE_QUALITY) {

        interpFilt=0;
	if(flags&RESAMPLE_BEST) interpFilt=1;

        if (factor > 1) {      /* SrcUp() is faster if we can use it */
            outBytes=SrcUp(Ltmp,OtmpS,factor,&Time,insize,Nwing,LpScl,Imp,ImpD,interpFilt);
            if (inchannels==2)
              outBytes=SrcUp(Rtmp,(short *)((char *)OtmpS+outBytes),factor,&Time2,insize,Nwing,LpScl,Imp,ImpD,
                         interpFilt) << 1;
        }
        else {
            outBytes=SrcUD(Ltmp,OtmpS,factor,&Time,insize,Nwing,LpScl,Imp,ImpD,interpFilt);
            if (inchannels==2)
              outBytes=SrcUD(Rtmp,(short *)((char *)OtmpS+outBytes),factor,&Time2,insize,Nwing,LpScl,Imp,ImpD,
                         interpFilt) << 1;
        }  
  
  
  } else {
  /* Fast */
        outBytes=SrcLinear(Ltmp,OtmpS,factor,&Time,insize);
        if (inchannels==2)
          outBytes=SrcLinear(Rtmp,(short *)((char *)OtmpS+outBytes),factor,&Time,insize) << 1;
  
  }
  
  
  /* Put the sample data back into the output buffer and free temp buffers */
/*  free(ItmpS);*/

    outCount=outBytes;
    Ltmp=OtmpS;
    Rtmp=Ltmp+outBytes;
    outC=(unsigned char *)outdata;
    outS=(short *)outdata;

    while(outCount) {
      
      sample=*Ltmp;
      if(inbits==8) {
         sample=sample16to8(sample);
         *outC=sample;
      } else {
         *outS=sample;
      }
      
      if(inchannels==2) {
      	sample=*Rtmp;
	if(inbits==8) {
           sample=sample16to8(sample);
           *outC=sample;
        } else {
           *outS=sample;
        }
      
      }
      outCount--;
       Ltmp++;
       Rtmp++;
       outC++;
       outS++;
    
    
    }
  
  free(OtmpS);
  
  return outBytes;


}

#include "resample/resample_stripped_josmith3.c"
