#include <e32std.h>

#include "SSSDThreadHandler.h"

extern "C" {
#include "sssd.h"
#include "internals.h"
#include "symbian.h"
}


/*
   Stop sssd system: turn off all playing sounds, kill thread, close
   device.  Returns -1 on error.
*/

int sssd_stopAudio( struct sssdaudio* ga ) {

    if( ga->audio_fd == -1 ) {
        ga->lastERR=ERR_NOT_INITIALIZED;
        return( ERR_NOT_INITIALIZED );
    }
    
    ga->lastERR = 0;

    sssd_stopBackground(ga);
    
    if (ga->max_voices) {		// JH thread was not created if max_voices is 0
		
        CSSSDThreadHandler *handler = (CSSSDThreadHandler *)ga->platformData;
        delete handler;
        ga->platformData = NULL;
    
    }

	/* Free Sample Memory */
	sssd_freeSamples(ga);

    /* Close device */
    ga->audio_fd = -1;


    /* free play buffer */
    if(ga->playBuffer!=NULL) {
       free(ga->playBuffer);
       ga->playBuffer=NULL;
    }

    /* Free Sample structs */
    if(ga->samples!=NULL) {
       free(ga->samples);
       ga->samples=NULL;
    }

    /* Free Playing structs */
    if(ga->playing!=NULL) {
       free(ga->playing);
       ga->playing=NULL;
    }

    /* destroy mutex */
    DELETE_MUTEX(ga);   

    return( ga->lastERR );

}
