
#include <stdio.h>
#include <stdlib.h>

#ifndef WIN32
   #include <unistd.h>
#else
   #define usleep(a) _sleep(a/1000);
#endif

#ifndef __ELATE__
#include <sssd.h>     /* libsssd defines */
#else
#include <amiga/sssd.h>
#endif

#include "loadSimpleWave.h"  /* routine to load simple Wav file */

void printMenu()
{
    printf( "Command Menu:\n"
            "\t? - print menu\n"
            "\t1 - play sample 1\n"
            "\t2 - play sample 2\n"
            "\t3 - play sample 3\n"
            "\t4 - play sample 4\n"
            "\t5 - play sample 1 with only left channel loaded\n"
            "\t6 - play sample 2 with only right channel loaded\n"
            "\tl - play sample 1 in a loop\n"
            "\tL - play sample 2 in a loop\n"
            "\tp - play sample 3 and 4 panning\n"
            "\tv - play sample 1 changing volume\n"
            "\tV - play sample 1 changing volume (using array)\n"
            "\tP - play sample 3 changing volume and panning\n" 
            "\tM - mute audio\n"
            "\tU - un-mute audio\n"
            "\ts - stop all samples\n"
            "\tm - start/stop background music (requires Sample.xxx file)\n"
            "\tt - print time of background music playback\n"
            "\tq - quit demo\n\n" );
}


int main(int argc, char **argv)
{
    int c,i,j;

    /* libsssd handle */
    struct sssdaudio *ga;

    /* sample references */
    int fire_id;
    int appear_id;
    int left_id;
    int right_id;
    int fire_left_id;
    int appear_right_id;
    int parmArray[4];
    
    /* wave file info */
    waveFile *waveData;

    
    printf("Init control struct...(%d)\n",DEFAULT_NUMBUFFERS);
    ga=sssd_initAudio(2,		/* Channels: 1=mono, 2=stereo */
    		 22050,		/* Frequency */
		 16,		/* bits per sample (8|16) */
		 4,		/* Maximum simul voices */
		 20,		/* Maximum samples */
		 0,             /* Size of audio hardware PCM buffer  0=let system decide*/
		 DEFAULT_NUMBUFFERS, /* Number of audio hardware bufffers */
		 0);		/* Flags */

    /* open soundcard device */
    printf("Opening device ...\n");
    if( sssd_startAudio( ga, DEFAULT_DEVICE ) < 0 )
        exit(-1);

    /* pre-cache samples ... keeping id's */
    printf( "Precaching samples ...\n" );

    waveData=loadSimpleWave("Fire.wav");
    if(waveData==NULL) { 
       printf("Could not load fire.wav\n");
       exit(-1);
    }

   #define RESAMPLE_TYPE RESAMPLE_FAST
    
    fire_id   = simpleWave2copySample( ga, waveData, RESAMPLE_TYPE);
    free(waveData);
    if(sssd_isError(fire_id)) {
       printf("ERROR copy registering sample (%d)\b\n",fire_id);
       exit(-1);
    }
    
    waveData=loadSimpleWave("Appear.wav");
    appear_id = simpleWave2copySample( ga, waveData, RESAMPLE_TYPE);
    free(waveData);
    
    waveData=loadSimpleWave("Left.wav");
    left_id   = simpleWave2copySample( ga, waveData, RESAMPLE_TYPE);
    free(waveData);
    
    waveData=loadSimpleWave("Right.wav");
    right_id  = simpleWave2copySample( ga, waveData, 0);
    free(waveData);
    
    waveData=loadSimpleWave("Fire.wav");
    fire_left_id   = sssd_copySampleLeft( ga, waveData->PCMdata, waveData->channels, waveData->freq,waveData->bits,waveData->chunkSize,0);
    free(waveData);
    
    waveData=loadSimpleWave("Appear.wav");
    appear_right_id = sssd_copySampleStereo( ga, waveData->PCMdata, waveData->channels, waveData->freq,waveData->bits,waveData->chunkSize, COPY_RIGHT);
    free(waveData);

    /* User interface loop */
    printMenu();
    c = 0;
    while( c != 'q' )
    {
        c = getchar();
        switch( c )
        {
            case '1':
                printf( "Playing fire sound once ...\n" );
                sssd_playSample( ga, fire_id, 1, 256, 0 );
                break;

            case '2':
                printf( "Playing appear sound once ...\n" );
                sssd_playSample( ga, appear_id, 1, 256, 0 );
                break;

            case '3':
                printf( "Playing left sound once ...\n" );
                sssd_playSample( ga, left_id, 1, 256, 0 );
                break;

            case '4':
                printf( "Playing right sound once ...\n" );
                sssd_playSample( ga, right_id, 1, 256, 0 );
                break;

            case '5':
                printf( "Playing fire sound (left channel) once ...\n" );
                sssd_playSample( ga, fire_left_id, 1, 256, 128 );
                break;

            case '6':
                printf( "Playing appear sound (right channel) once ...\n" );
                sssd_playSample( ga, appear_right_id, 1, 256, 128 );
                break;

            case 'm':
                if( sssd_checkBackground( ga ) )
                {
                    printf( "Stopping background MP3 ...\n" );
                    sssd_stopBackground( ga );
                }
                else
                {
                    printf( "Starting background MP3 ...\n" );
                    sssd_playBackground( ga, "Sample.mp3", 0, "" );
                }
                break;

            case 't':
                printf( "Background time is %.2f sec.\n", sssd_timeBackground(ga));
                break;

            case 'l':
                printf( "Playing fire sound in a loop ...\n" ); 
                sssd_playSample( ga, fire_id, 0, 256, 0 );
                break;

            case 'L':
                printf( "Playing appear sound in a loop ...\n" ); 
                sssd_playSample( ga, appear_id, 0, 256, 0 );
                break;

            case 'p':
                printf( "Playing left and right sound panning ... please wait.\n" ); 
                sssd_playSample( ga, left_id, 0, 256, -128 );
                parmArray[0]=CTRL_S_PAN;
		parmArray[2]=CTRL_S_END;
		for (i=-128; i<=128; i += 16) {
                 printf ("Pan value set to %i\n",i);
		 parmArray[1]=i;
                 sssd_ctrlSample ( ga, left_id, parmArray);
                 usleep (500000);
                }
                usleep (1000000);
                sssd_stopSample( ga, left_id );
                sssd_playSample( ga, right_id, 0, 256, 0 );
                parmArray[0]=CTRL_S_PAN;
		parmArray[2]=CTRL_S_END;
                for (i=-128; i<=128; i += 16) {
                 printf ("Pan value set to %i\n",i);
		 parmArray[1]=i;
                 sssd_ctrlSample ( ga, right_id, parmArray);
                 usleep (500000);
                }
                usleep(1000000);
                sssd_stopSample( ga, right_id );
                break;

            case 'v':
                printf( "Playing fire sound changing volume ... please wait.\n" ); 
                sssd_playSample( ga, fire_id, 0, 256, 0 );
                parmArray[0]=CTRL_S_VOLUME;
		parmArray[2]=CTRL_S_END;
                for (i=0; i<=256; i += 16) {
                 printf ("Volume value set to %i\n",i);
		 parmArray[1]=i;
                 sssd_ctrlSample ( ga, fire_id, parmArray);
                 usleep (500000);
                }
                usleep (1000000);
                sssd_stopSample( ga, fire_id );
                break;

            case 'V':
                printf( "Playing fire sound changing volume (array) ... please wait.\n" ); 
                parmArray[0]=CTRL_S_VOLUME;
		parmArray[2]=CTRL_S_END;
		sssd_playSample( ga, fire_id, 0, 256, 0 );
                for (i=0; i<=256; i += 16) {
                 parmArray[1]=i;
		 printf ("Volume value set to %i\n",i);
                 sssd_ctrlSample ( ga, fire_id, parmArray);
                 usleep (500000);
                }
                usleep (1000000);
                sssd_stopSample( ga, fire_id );
                break;

            case 'P':
                printf( "Playing left sound changing volume and panning ... please wait.\n" ); 
                sssd_playSample( ga, left_id, 0, 256, 0 );
                parmArray[0]=CTRL_S_VOLUME;
		parmArray[2]=CTRL_S_PAN;
		parmArray[4]=CTRL_S_END;
		for (i=-128; i<=128; i += 8) {
                 if (i<0) { 
                  j=2*(i+128); 
                 } else {
                  j=256-2*(i); 
                 } 
                 printf ("Pan value set to %i, Volume value set to %i\n",i,j);
		 parmArray[1]=j;
		 parmArray[3]=i;
                 sssd_ctrlSample ( ga, left_id, parmArray);
                 usleep (500000);
                }
                usleep (1000000);
                sssd_stopSample( ga, left_id );
                break;

            case 's':
                printf( "Stopping all effects ...\n" );
                sssd_stopAllSamples( ga );
                break;

            case 'M':
                printf( "Mute sound\n" );
                sssd_pauseAudio( ga );
                break;

            case 'U':
                printf( "Un-mute sound\n" );
                sssd_unpauseAudio( ga );
                break;

            case 'h':
            case '?':
                printMenu();
                break;
        }
    }

    /* Shut audio down */
    sssd_stopAudio( ga ); 
    sssd_freeAudio( ga );
    
    return( 0 );
}


/*EOF*/
